@@ -104,7 +104,7 @@ except ImportError:
                 " install ordereddict 1.1 from pypi for python2.6")
 
 __pkgname__ = 'python-crontab'
-__version__ = '2.2.3'
+__version__ = '2.2.4'
 
 ITEMREX = re.compile(r'^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)'
                      r'\s+([^@#\s]+)\s+([^#\n]*)(\s+#\s*([^\n]*)|$)')
@@ -167,6 +167,7 @@ def open_pipe(cmd, *args, **flags):
     a. keyword args are flags and always appear /before/ arguments for bsd
     """
     cmd_args = tuple(shlex.split(cmd))
+    env = flags.pop('env', None)
     for (key, value) in flags.items():
         if len(key) == 1:
             cmd_args += ("-%s" % key),
@@ -175,7 +176,7 @@ def open_pipe(cmd, *args, **flags):
         else:
             cmd_args += ("--%s=%s" % (key, value)),
     args = tuple(arg for arg in (cmd_args + tuple(args)) if arg)
-    return sp.Popen(args, stdout=sp.PIPE, stderr=sp.PIPE)
+    return sp.Popen(args, stdout=sp.PIPE, stderr=sp.PIPE, env=env)
 
 def _unicode(text):
     """Convert to the best string format for this python version"""
@@ -353,16 +354,16 @@ class CronTab(object):
     def run_scheduler(self, timeout=-1, **kwargs):
         """Run the CronTab as an internal scheduler (generator)"""
         count = 0
-        self.run_pending()
         while count != timeout:
-            count += 1
-            sleep(kwargs.get('cadence', 60))
             now = datetime.now()
             if 'warp' in kwargs:
                 now += timedelta(seconds=count * 60)
             for value in self.run_pending(now=now):
                 yield value
 
+            sleep(kwargs.get('cadence', 60))
+            count += 1
+
     def render(self):
         """Render this crontab as it would be in the crontab."""
         crons = [unicode(cron) for cron in self.lines]
@@ -651,18 +652,20 @@ class CronItem(object):
         """Runs the command if scheduled"""
         now = now or datetime.now()
         if self.is_enabled():
-            if self.last_run is not None:
-                next_time = self.schedule(self.last_run).get_next()
-                if next_time < now:
-                    self.last_run = now
-                    return self.run()
-            else:
+            if self.last_run is None:
                 self.last_run = now
 
+            next_time = self.schedule(self.last_run).get_next()
+            if next_time < now:
+                self.last_run = now
+                return self.run()
+
     def run(self):
         """Runs the given command as a pipe"""
+        env = os.environ.copy()
+        env.update(self.env.all())
         shell = self.env.get('SHELL', SHELL)
-        (out, err) = open_pipe(shell, '-c', self.command).communicate()
+        (out, err) = open_pipe(shell, '-c', self.command, env=env).communicate()
         if err:
             LOG.error(err.decode("utf-8"))
         return out.decode("utf-8").strip()
